;Misst die Spannung vom Hall-Element und sendet den Wert ber UART an PC 
;Es gibt 4 Mess-Ranges, die ber UART-RX ausgewhlt werden
;;--------------------------------------------------------------------------
;; Connections
;;  Messeingang Spannung - AN7, Pin7
;;  Mess-Range-Auswahl A - RC5, Pin5
;;  Mess-Range-Auswahl B - RC4, Pin6
;;  UART Tx              - RA1, Pin12
;;  UART Rx              - INT, Pin11 
;;--------------------------------------------------------------------------
;;
;; TMR0 wird fr Sample-Rate des gerts verwendet. Pre-scale = 1:16 - 234 Ticks = 1,25ms (UART Transfer von 2 Byte dauert ca. 1,1ms) 
;; Clock wird durch externen Quarz gemacht. Takt ist ca. 12MHz.
;; Verwendet UART mit Baud-Rate 19,2kBaud

;;Config-register sollte bei Programmierung auf 00 0001 1000 0010 (0182) stehen

	processor  16F676
	radix  DEC
	__CONFIG 0x0182

;Register Adressen
INDF        EQU   0x00
TMR0        EQU   0x01
STATUS      EQU   0x03
FSR         EQU   0x04
PORTA       EQU   0x05
PORTC       EQU   0x07
INTCON      EQU   0x0B
PIR1	    EQU   0x0C
TMR1L       EQU   0x0E
TMR1H       EQU   0x0F
T1CON       EQU   0x10
CMCON		EQU   0x19
ADRESH      EQU   0x1E
ADCON0      EQU   0x1F
OPTIONR     EQU   0x81
TRISA       EQU   0x85
TRISC       EQU   0x87
PIE1        EQU   0x8C
PCON        EQU   0x8E
OSCCAL      EQU   0x90
ANSEL       EQU   0x91
WPUA	    EQU   0x95
IOCA	    EQU   0x96
VRCON	    EQU   0x99
EEDAT	    EQU   0x9A
EEADR	    EQU   0x9B
EECON1	    EQU	  0x9C
EECON2      EQU   0x9D
ADRESL      EQU   0x9E
ADCON1      EQU   0x9F


_C      	EQU 	0		;Carry-Bit
_Z      	EQU   	2		;Zero-Bit
_W			EQU		0		;Destination W bei z.B. INCF
_R			EQU		1		;Destination selbes Register bei z.B. INCF 

#define	_BANK0	BCF STATUS,5	;Umschalten auf Bank0
#define	_BANK1	BSF STATUS,5	;Umschalten auf Bank1
#define	_RSTX	PORTA,1			;UART Tx
#define	_RSRX	PORTA,2			;UART Rx (=INT)
#define	_A		PORTC,5			;Channel-Select A
#define	_B		PORTC,4			;Channel-Select B
#define _ADCGO	ADCON0,1		;Go-Bit fr ADC
#define _GIE	INTCON,7		;Global Interrupt enable
#define _INTF	INTCON,1		;Interrupt-Flag UART-RX
#define _CARRY	STATUS,0		;CArry-Bit in Status


;Memory
MEM         EQU   0x20
temp		EQU	  MEM			;Ntig fr UART-Routinen
n			EQU	  MEM+1			;Ntig fr UART-Routinen
m			EQU   MEM+2			;Ntig fr UART-Routinen
DataL		EQU	  MEM+3
DataH		EQU   MEM+4
INTWSAVE	EQU	  MEM+62
INTSSAVE	EQU	  MEM+63


;******* Programm Start **************************************************
	ORG	0
		CLRF  	PORTA			;Setzt UART-TX auf low, d.h. Zustand vor Startbit
		CLRF  	PORTC			;Setzt A und B auf low, d.h. Strommessung-Modus
 		GOTO	main

;******* Interrupt Service Routine ***************************************
	ORG	4
;Mgliche Interrupt-Quellen und ihre Funktion
;- INT steigende Flanke: UART-Empfang vom PC starten
ISR		BCF		_GIE			;Alle Interrupts disablen
		MOVWF	INTWSAVE		;preserve w register
		SWAPF	STATUS,_W		;preserve status register
		MOVWF	INTSSAVE
ISRchk	;Prfung, was den Int ausgelst hat
		BTFSS	_INTF			;INT steigende Flanke?
		GOTO	ISRend			;Nein: ISR ignorieren
		BCF		_RSTX			;Ja: Falls transmit unterbrochen wurde, wird dieser durch dieses RX korrupiert. Sicherstellen, dass waehrend RX nur High/0V (Stop-Bit) gesendet wird 		
		CALL	RSrx			;UART lesen
		MOVWF	PORTC			;PORTC entsprechend setzen - beeinflusst nur RC4 und RC5 (= A und B)
		MOVLW	0x01			;Die Schleifenzaehler m und n (RS-Schleife und baud) sind jetzt beide auf 0. Falls ein Transmit unterbrochen wurde, soll dieser nicht mit 0 weitergehen, da das nchste DECFSZ das dann auf 255 setzen wrde - das waere dann langer Transmit bzw. lange baud-Pause 
		MOVWF	m
		MOVWF	n
		MOVLW	0xFF			;Daten eines evtl unterbrochenen Transmit auf High/0V setzen, damit kein nach-gesendetes Bit flschlicherweise als Start-Bit interpretiert wird
		MOVWF	DataL
		MOVWF	DataH
		MOVWF	temp
		BCF		_INTF			;Flag zurcksetzen - erst nach dem RSrx, weil ansonsten ein Bitwechsel in der bertragung das Flag wieder setzen und einen weiteren Interrupt auslsen wrde
ISRend	SWAPF	INTSSAVE,_W		;restore STATUS register
		MOVWF	STATUS
		SWAPF	INTWSAVE,_R		;restore w register
		SWAPF	INTWSAVE,_W
		RETFIE					;enabled gleichzeitig auch wieder alle Interrupts


;***********Main ********************************************
main	;Prozessor Init
		MOVLW	0x10
		MOVWF 	INTCON		;GIE noch disabled, INTE enabled	
		MOVLW	0x00
		MOVWF 	T1CON		;TMR1:disabled
		MOVLW	0x07
		MOVWF 	CMCON		;Comparator:disabled
		MOVLW 	0x9D		;ADC: Right justified, VRef=Vdd, Channel=7, ADC an aber nocht nicht aktiv
		MOVWF 	ADCON0
		_BANK1				;Umschalten auf Bank 1
		MOVLW 	0xC3		;Options: INT auf steigende Flanke (Startbit), TMR0 prescale=1:16		
		MOVWF 	OPTIONR
		MOVLW 	0xFD		
		MOVWF 	TRISA		;PortA: Bis auf RA1 (UART-TX) alles Inputs
		MOVLW	0xCF
		MOVWF 	TRISC		;PortC: Bis auf C4 und C5 (A und B) alles Input
		CLRF  	PIE1		;Keine Peripherie-Interruptrs
		MOVLW	0x80
		MOVWF 	ANSEL		;Nur AN7 in ADC-Pin
		CLRF	WPUA		;Keine Pull-ups
		CLRF	IOCA		;Keine Interrupt-on change auf PortA (INT ist separat)
		CLRF	VRCON		;Voltage Reference off
		MOVLW 	0x20
		MOVWF 	ADCON1		;ADCON1: 32 Tosc bei 12MHz = 2,6us Conversion Zeit
		_BANK0 				;Bank 0
		BSF		_GIE		;Interrupts enablen	

		;****** Hauptroutine **************
		;Macht Messung und gibt die dann ueber UART aus
		MOVLW	250
		MOVWF	TMR0
MLoop	MOVF	TMR0,_W		;Warte bis TMR0=0
		BTFSS	STATUS,_Z	
		GOTO	$-2
		MOVLW	22			;TMR0 neu initialisieren 234 Ticks (=1,25ms) bis 0 -> Sample-Rate des Gerts
		MOVWF	TMR0
		CALL 	baud		;Eigentlich ist keine acquisition time ntig, weil nicht zw. AD-Eingngen umgeschaltet wird - trotzdem warte 1 Baud-Zeit
		BSF   	_ADCGO		;Starte ADC Conversion
		BTFSC 	_ADCGO		;Pruefe, ob Konvertierung beendet (GO=0)
		GOTO  	$-1			;Wenn nein, weiterwarten
		MOVF  	ADRESH,_W	;Wenn ja, ADRESH...	
		MOVWF 	DataH		;...nach DataH speichern
		_BANK1			 	;Umschalten auf Bank 1
		MOVF  	ADRESL,_W	;ADRESL...
		_BANK0			 	;Umschalten auf Bank 0
		MOVWF 	DataL		;...nach DataL speichern
		RLF		DataL,_W	;MSB des Low-Bytes ins LSB des High-Bytes schieben
		RLF		DataH,_R
		BSF		DataH,7		;MSB des DataH wird durch 1 markiert (war vorher sicher 0)
		BCF		DataL,7		;MSB des DataL wird durch 0 markiert (vorheriges MSB ist jetzt LSB in DataH)
		MOVF	DataH,_W	;DataH ueber UART senden
		CALL 	RStx
		MOVF	DataL,_W	;DataL ueber UART senden
		CALL	RStx
		GOTO	MLoop


;;******* RStx *******************************************************
;Funktion: Sendet das Byte in W ber RS232 Protokoll(negative Logik!)
;Startbit(Low=5V), 8 Datenbits(Low=5V, High=0V) LSB first, Stopbit(High=0V) // Baudrate je nach delay in baud-Routine, jeder Wechsel zwischen zwei Bits dauert 10 Zyklen + 1*baud, No Parity
;Ein-/Ausgangswerte: W beinhaltet zu sendendes Byte
;Bentigte Subroutinen: baud
;Bentigte Variablen: temp, n, m
;Bentigte Register: keine
;Bentigte Defines: _RSTX, _CARRY (Transmit-Pin, Carry-Bit in STATUS)
;Unverndert bleibt: keine
;Verndert werden  : temp, n, m
RStx	MOVWF	temp			;W nach temp
		MOVLW 	.8
		MOVWF 	n				;8 Bits senden
		BSF   	_RSTX			;Startbit setzen (Low = 5V)
		NOP
		NOP
		NOP
		NOP
		NOP
		CALL  	baud			;Baudzeit warten
		RRF   	temp,1			;Bits durchschieben
		BTFSS 	_CARRY			;Bit checken = 1?
		GOTO	$+4				;nein: Sende 0
		NOP						;ja: Sende 1
		BCF	  	_RSTX			;High=0V
		GOTO	$+4				;Ab zur Pause 
		BSF   	_RSTX			;Low=5V
		NOP
		NOP
		CALL  	baud			;Baudzeit warten
		DECFSZ 	n,1				;n=n-1, n=0 (Byte zu Ende)?
		GOTO  	$-11			;nein: weiter mit naechstem Bit
		NOP						;ja: Stopbit setzen
		NOP
		NOP
		NOP
		NOP
		BCF   	_RSTX			;Stopbit setzen (High = 0V)
		CALL  	baud
		CALL	baud
		RETURN


;;******* RSrx *******************************************************
;Funktion: Wartet auf ein Startbit und liest dann ein Byte ber RS-232 Protokoll nach W (negative Logik! - ohne externe Inverter)
;Startbit(Low=5V), 8 Datenbits(Low=5V, High=0V) LSB first, Stopbit(High=0V) // Baudrate je nach delay in baud-Routine, jeder Wechsel zwischen zwei Bits dauert 10 Zyklen + 1*baud, No Parity
;Ein-/Ausgangswerte: W beinhaltet empfangenes Byte
;Bentigte Subroutinen: baud, baud2
;Bentigte Variablen: temp, n, m
;Bentigte Register: keine
;Bentigte Defines: _RSRX, _CARRY (Receive-Pin, Carry-Bit in STATUS)
;Unverndert bleibt: keine
;Verndert werden  : temp, n, m		
RSrx	CLRF	temp
		MOVLW	.8
		MOVWF	n
		NOP
		NOP
		NOP
		NOP
		CALL	baud2		;Startbit wurde registriert, warte 1/2 baudzeit (danach gleich noch eine ganze) = 1,5 baud seit der Startbitflanke = Mitte vom ersten bit
		CALL	baud		;Warte volle baudrate-10 Zyklen (werden in Schleife verbraten), um in naechstes Bit zu kommen
		NOP
		BTFSC	_RSRX		;Pruefe Bit
		GOTO	$+3			;=1: (=Logisch 0)
		BSF		_CARRY		;=0: (=Logisch 1) Carry mit 1 besetzen
		GOTO	$+3
		BCF		_CARRY		;Carry mit 0 besetzen
		NOP					;Damit gleich lang wie BCF-Zweig
		RRF		temp,1		;Carry in Byte verschieben (RS232:LSB first -> Bits werden von links reingeschoben)
		DECFSZ	n,1
		GOTO	$-10
		CALL	baud		;Sicherstellen, dass Stopbit vor nchstem Receive empfangen wird
		MOVF	temp,0		;gelesenes Byte nach W
		RETURN
	

;******** baud **********************
;Funktion: Warteschleife fuer Baudrate 19200 (=52us) bei 12MHz/4 Takt (=156 Zyklen)
;10 Zyklen werden schon in der RS232 Routine zwischen jedes Bit gesetzt d.h. zusammen mit aufrufendem CALL werden hier 146 Zyklen verbraten
;2+2+(46*3)+1+1+2 (CALL + 2*MOV + 46*Schleife + NOP + NOP + RETURN)
;Ein-/Ausgangswerte: keine
;Bentigte Subroutinen: keine
;Bentigte Variablen: m
;Bentigte Register: keine
;Bentigte Defines: keine
;Unverndert bleibt: keine
;Verndert werden  : m
baud	MOVLW  	.46	
		MOVWF  	m
		DECFSZ 	m,1			
		GOTO  	$-1
		NOP
		NOP
		NOP
		RETURN


;******** baud2 **********************
;Funktion: Warteschleife fuer 1/2 Baudrate 19200 (=26us) bei 12MHz/4 Takt (=78 Zyklen). 
;Es werden keine Zwischentakte bercksichtigt d.h. zusammen mit aufrufendem CALL werden hier 78 Zyklen verbraten
;2+2+(24*3)+1+2 (CALL + 2*MOV + 24*Schleife + RETURN)
;Ein-/Ausgangswerte: keine
;Bentigte Subroutinen: keine
;Bentigte Variablen: m
;Bentigte Register: keine
;Bentigte Defines: keine
;Unverndert bleibt: keine
;Verndert werden  : m	
baud2	MOVLW  	.24	
		MOVWF  	m
		DECFSZ 	m,1			
		GOTO  	$-1
		NOP
		RETURN
			


		END

